@@ -1,7 +1,8 @@
 import asyncio
-from copy import copy
-from datetime import datetime
 import logging
+from copy import copy
+from enum import Enum
+from datetime import datetime, timezone
 
 from .holder import TaskHolder
 from .task import register
@@ -10,6 +11,13 @@ from .task import register
 log = logging.getLogger(__name__)
 
 
+class JoinStatus(Enum):
+
+    RUNNING = 'running'
+    TIMEOUT = 'timeout'
+    DONE = 'done'
+
+
 @register('join', 'execute')
 class JoinTask(TaskHolder):
 
@@ -19,23 +27,25 @@ class JoinTask(TaskHolder):
     The `wait_for` config parameter is mandatory and is a list of task IDs.
     """
 
+    __slots__ = ('_data_stash', '_wait_for', '_report', '_task', 'data')
+
     def __init__(self, config):
         super().__init__(config)
         self._data_stash = []
         # Must be either a number of tasks or a list of ids
         # Copy because we alter the list/int during `_step`
         self._wait_for = copy(self.config['wait_for'])
-        # Default timeout to avoid infinite wait
-        self._timeout = self.config.get('timeout')
 
         # Reporting
-        self._report = {'tasks': [], 'status': 'running'}
+        self._task = None
+        self.data = None
+        self._report = {'tasks': [], 'status': JoinStatus.RUNNING.value}
 
     def report(self):
         return self._report
 
     def add_task_report(self, task_id):
-        task_report = {'id': task_id, 'end_time': datetime.utcnow().isoformat()}
+        task_report = {'id': task_id, 'end_time': datetime.now(timezone.utc)}
         self._report['tasks'].append(task_report)
         self._task.dispatch_progress(self._report)
 
@@ -63,25 +73,22 @@ class JoinTask(TaskHolder):
                 return
 
     async def execute(self, event):
-        log.info(
-            'Join task waiting for tasks (%s) (timeout: %s)',
-            self._wait_for, self._timeout
-        )
-        self._task = asyncio.Task.current_task()
+        log.info('Join task waiting for tasks (%s)', self._wait_for)
+        self._task = asyncio.current_task()
         # Trigger first step for this event
         self._step(event)
-        data = event.data
-        try:
-            await asyncio.wait_for(self._wait_for_tasks(), self._timeout)
-        except asyncio.TimeoutError:
-            log.warning("Join timed out, still waiting for %s", self._wait_for)
-            self._report['status'] = 'timeout'
-            self._task.dispatch_progress(self._report)
-        else:
-            log.debug('All awaited parents joined')
-            self._report['status'] = 'done'
-            self._task.dispatch_progress(self._report)
+        self.data = event.data
+        await self._wait_for_tasks()
+        log.debug('All awaited parents joined')
+        self._report['status'] = JoinStatus.DONE.value
+        self._task.dispatch_progress(self._report)
         # Data contains the outputs of the first task to join
         # Variable `data_stash` contains a list of all parent tasks outputs
-        data['data_stash'] = self._data_stash
-        return data
+        self.data['data_stash'] = self._data_stash
+        return self.data
+
+    def teardown(self):
+        self._report['status'] = JoinStatus.TIMEOUT.value
+        self._task.dispatch_progress(self._report)
+        self.data['data_stash'] = self._data_stash
+        return self.data
