@@ -2,7 +2,6 @@ import logging
 from uuid import uuid4
 
 from .task import new_task
-from tukio.event import Event
 from tukio.utils import Listen
 
 
@@ -18,10 +17,13 @@ class TaskTemplate:
     `asyncio.Task`) and provide execution report.
     """
 
-    def __init__(self, name, uid=None, config=None, topics=[]):
+    __slots__ = ('name', 'config', 'topics', 'timeout', 'uid')
+
+    def __init__(self, name, uid=None, config=None, timeout=None, topics=[]):
         self.name = name
         self.config = config or {}
         self.topics = topics
+        self.timeout = timeout
         self.uid = uid or str(uuid4())
 
     @property
@@ -32,10 +34,13 @@ class TaskTemplate:
         """
         Create a new asyncio task from the current task template.
         """
-        task = new_task(self.name, data=data, config=self.config, loop=loop)
-        task._template = self
-        task.inputs = data.data if isinstance(data, Event) else data
-        return task
+        return new_task(
+            self.name,
+            data=data,
+            config=self.config,
+            timeout=self.timeout,
+            loop=loop,
+        )
 
     @classmethod
     def from_dict(cls, task_dict):
@@ -67,7 +72,8 @@ class TaskTemplate:
         name = task_dict['name']
         config = task_dict.get('config', {})
         topics = task_dict.get('topics', [])
-        return cls(name, uid=uid, config=config, topics=topics)
+        timeout = task_dict.get('timeout')
+        return cls(name, uid=uid, config=config, timeout=timeout, topics=topics)
 
     def as_dict(self):
         """
@@ -75,9 +81,13 @@ class TaskTemplate:
         task template is linked to a task execution object, the dictionary
         contains the execution (stored at key 'exec').
         """
-        task_dict = {"name": self.name, "id": self.uid}
-        task_dict.update({"config": self.config, "topics": self.topics})
+        task_dict = {'name': self.name, 'id': self.uid}
+        task_dict.update({
+            'config': self.config,
+            'timeout': self.timeout,
+            'topics': self.topics,
+        })
         return task_dict
 
     def __str__(self):
-        return "<TaskTemplate name={}, uid={}>".format(self.name, self.uid)
+        return '<TaskTemplate name={}, uid={}>'.format(self.name, self.uid)
