@@ -2,7 +2,6 @@ from enum import Enum
 
 
 class SkipTask(Exception):
-    pass
 
     def __init__(self, reason=None):
         self.reason = reason
@@ -27,6 +26,7 @@ class FutureState(Enum):
     finished = 'finished'
     skipped = 'skipped'
     suspended = 'suspended'
+    timeout = 'timeout'
 
     @classmethod
     def get(cls, future):
@@ -38,6 +38,8 @@ class FutureState(Enum):
         if not future.done():
             return cls.pending
         if future.cancelled():
+            if hasattr(future, 'timed_out') and future.timed_out is True:
+                return cls.timeout
             return cls.cancelled
         if future._exception:
             if isinstance(future._exception, SkipTask):
@@ -46,7 +48,9 @@ class FutureState(Enum):
         return cls.finished
 
     def done(self):
-        return self in (self.finished, self.skipped, self.exception)
+        return self in (
+            self.finished, self.skipped, self.exception, self.timeout
+        )
 
 
 class Listen(Enum):
@@ -59,9 +63,9 @@ class Listen(Enum):
         'topics': receive data dispatched only in template's topics
     """
 
-    everything = "everything"
-    nothing = "nothing"
-    topics = "topics"
+    everything = 'everything'
+    nothing = 'nothing'
+    topics = 'topics'
 
     @classmethod
     def get(cls, topics):
@@ -76,3 +80,33 @@ class Listen(Enum):
             return cls.topics
         else:
             raise TypeError("'{}' is not a list".format(topics))
+
+
+class TimeoutHandle:
+
+    """
+    Register a timeout on a given task to cancel its execution.
+    This does not rely on the creation of a new task.
+    """
+
+    __slots__ = ('task', 'timeout', 'handle')
+
+    def __init__(self, task, timeout):
+        self.task = task
+        self.timeout = timeout
+        self.handle = None
+
+    def _timeout_task(self):
+        self.task.timeout()
+        self.handle = None
+
+    def _end_task(self, future):
+        if self.handle is not None:
+            self.handle.cancel()
+            self.handle = None
+
+    def start(self):
+        self.task.add_done_callback(self._end_task)
+        self.handle = self.task._loop.call_later(
+            self.timeout, self._timeout_task
+        )
